/**
 * @author  Tilen Majerle
 * @email   tilen@majerle.eu
 * @website http://stm32f4-discovery.com
 * @link    http://stm32f4-discovery.com/2014/08/library-25-am2301-dht21-sensor-stm32f4xx/
 * @version v1.3
 * @ide     Keil uVision
 * @license GNU GPL v3
 * @brief   Basic library for DHT (DHT21) temperature and humidity sensor
 *	
@verbatim
   ----------------------------------------------------------------------
    Copyright (C) Tilen Majerle, 2015
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.
     
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
   ----------------------------------------------------------------------
@endverbatim
 */
#ifndef DHT_H
#define DHT_H 130

/* C++ detection */
#ifdef __cplusplus
extern "C" {
#endif

#include "tm_stm32f4_disco.h"
#include "arm_math.h"

//extern const char DHT_Result_Message[8][20];

double_t DHT_ComputeHeatIndex(double_t,double_t,int16_t);
float32_t DHT_ComputeDewPoint(float32_t,float32_t,int16_t);
float32_t DHT_GetComfortRatio(int16_t*,float32_t,float32_t,int16_t);
uint8_t DHT_ComputePerception(float32_t,float32_t,int16_t); 
uint8_t DHT_GetComfortProfile(float32_t,float32_t,int16_t);
double_t DHT_Compute_Altitude(float32_t p);
double_t DHT_ComputeGetPressureAtSeaLevel(float32_t pressure, float32_t altitude);

typedef struct msg_s 
{
  uint8_t cmd;
  uint8_t len;
  uint8_t data[8];
}
msg_t;

enum {Celsius=0, Fahrenheit=1};

// References https://en.wikipedia.org/wiki/Dew_point ==> Relationship to human comfort
enum PerceptionState
{
  DHT_Perception_Dry = 0,
  DHT_Perception_VeryComfy = 1,
  DHT_Perception_Comfy = 2,
  DHT_Perception_Ok = 3,
  DHT_Perception_UnComfy = 4,
  DHT_Perception_QuiteUnComfy = 5,
  DHT_Perception_VeryUnComfy = 6,
  DHT_Perception_SevereUncomfy = 7
};

// Reference: http://epb.apogee.net/res/refcomf.asp (References invalid)
enum ComfortState
{
  DHT_Comfort_OK = 0,
  DHT_Comfort_TooHot = 1,
  DHT_Comfort_TooCold = 2,
  DHT_Comfort_TooDry = 4,
  DHT_Comfort_TooHumid = 8,
  DHT_Comfort_HotAndHumid = 9,
  DHT_Comfort_HotAndDry = 5,
  DHT_Comfort_ColdAndHumid = 10,
  DHT_Comfort_ColdAndDry = 6
};

typedef struct
{
  //Represent the 4 line equations:
  //dry, humid, hot, cold, using the y = mx + b formula
  float32_t m_tooHot_m, m_tooHot_b;
  float32_t m_tooCold_m, m_tooHCold_b;
  float32_t m_tooDry_m, m_tooDry_b;
  float32_t m_tooHumid_m, m_tooHumid_b;	
}
ComfortProfile;

typedef struct _dht_computing_value_
{
 float32_t temperature,humidity,pressure,DewPoint,GetComfortRatio;
 double_t HeatIndex,Altitude;
 double_t PressureAtSeaLevel;
 uint8_t Perception,ComfortProfile;
 int16_t ComfortStatus,Type;
}
DHT_msg_t;

//void DHT_Compute_Propiertys(DHT_msg_t*, float32_t, float32_t, float32_t);
void DHT_Compute_Propiertys(DHT_msg_t*);

/* C++ detection */
#ifdef __cplusplus
}
#endif


#endif

