
/** Put this in the src folder **/

#include "cmsis_os2.h"
#include "i2c-lcd.h"
#include "rtx_os.h"
#include "defines.h"

#define	FAST_LCD
#define	I2C_BLOCK_TRANSFER

#define bitmask_one(x) ((uint8_t)(x)&0xf0)
#define bitmask_two(x) ((uint8_t)((x)<<4)&0xf0)

uint8_t backlight;

enum {cmd=0x00, dta=0x01, lcd_en=0x04, bckglight=0x08};

lcd_status_t lcd_control(uint8_t data, uint8_t lcd_rs)
{
	lcd_status_t state_t = (lcd_status_t)											//volatile lcd_status_t state_t = (lcd_status_t)
	 {
		 .data[0] = (bitmask_one(data) | backlight | lcd_en | lcd_rs),		//en=1 0x04,rs=1 0x01,	backlight=1	0x08
		 .data[1] = (bitmask_one(data) | backlight | lcd_rs), 						//en=0 0x04,rs=1 0x01,	backlight=1	0x08
		 .data[2] = (bitmask_two(data) | backlight | lcd_en | lcd_rs),		//en=1 0x04,rs=1 0x01,	backlight=1	0x08
		 .data[3] = (bitmask_two(data) | backlight | lcd_rs), 						//en=0 0x04,rs=1 0x01,	backlight=1	0x08
	 };
	return state_t;
}
void lcd_send_statement(uint8_t data,uint8_t statement)
{
	#ifndef I2C_BLOCK_TRANSFER
	uint8_t i;
	#endif
	lcd_status_t ctrl_word;
	switch(statement)
	 {
		case	cmd		: ctrl_word=lcd_control(data,cmd);	break;
		case	dta		:	ctrl_word=lcd_control(data,dta);	break;
	 }
	#ifdef I2C_BLOCK_TRANSFER
	i2c_reg_write_no_register((ADDRESS_PCF8574A | WRITE),(uint8_t*)&ctrl_word,sizeof(lcd_status_t));
	#else
	for(i=0x00; i<sizeof(lcd_status_t); i++) i2c_write_no_register(ADDRESS_PCF8574A | WRITE,ctrl_word.data[i]);
	#endif
}

void lcd_init(uint8_t lcd_backlight)
{
	backlight=lcd_backlight;
	lcd_send_statement(0x02,cmd);
	osDelay(Delay_5ms);
	lcd_send_statement(0x28,cmd);
	osDelay(Delay_5ms);
	lcd_send_statement(0x0c,cmd);
	osDelay(Delay_5ms);
	lcd_send_statement(0x80,cmd);		//Goto 0,0
	osDelay(Delay_5ms);
	lcd_send_statement(0x01,cmd);		//ClrScr
	osDelay(Delay_20ms);	
}

#ifdef FAST_LCD

void lcd_send_string(uint8_t *string)																									//Rutinka obcas prida/ubere jeden znak navyse a potom formatovany sprintf 04u nefunguje spravne ???, ale prisiel som na to, ze to robi malo pamati v STACK-u ...
{
	while(*string) lcd_send_statement((uint8_t)(*string++),dta);																	//Vypisuje sice programatorsky perfektne, ale neviem co to robi, ci nie chybu, while(*string) kontroluje '\0' !!! inac pokracuje slucka dalej !!! ....
	//while(*string!=(uint8_t)'\0') lcd_send_statement((uint8_t)(*string++),dta);								//Vypisuje sice programatorsky perfektne, ale neviem co to robi, ci nie chybu, while(*string) kontroluje '\0' !!! inac pokracuje slucka dalej !!! ....
}

#else

void lcd_send_string (uint8_t *string)
{
	uint8_t i;
	for(i=0; i<(LCD_COLUMNS-1); i++) 
	{
	 switch(string[i])
	  {
		 default		:	lcd_send_statement((uint8_t)string[i],dta);	break;
		 case	'\r'	:	
		 case	'\n'	:	
		 case	'\0'	:	i=LCD_COLUMNS;	break;
		}
	}
}

#endif

void lcd_string(uint8_t Line,uint8_t *string)
{
	lcd_send_statement(Line,cmd);
	osDelay(Delay_2ms);
	lcd_send_string(string); 
}

void Lcd_Refresh_Display(uint8_t Line,uint8_t *string,uint8_t lcd_backlight)
{
 backlight=lcd_backlight;
 switch(Line)
 {
	 case	0		:	Line=Riadok1; break;
	 case	1		:	Line=Riadok2; break;
	 case	2		:	Line=Riadok3; break;
	 case	3		:	Line=Riadok4; break;
	 default	:	Line=Riadok1; break;
 }
 lcd_send_statement(Line,cmd);
 osDelay(Delay_2ms);
 lcd_send_string(string);
 //while(*string) lcd_send_statement((uint8_t)*string++,dta);										//Vypisuje sice programatorsky perfektne, ale neviem co to robi, ci nie chybu, while(*string) kontroluje '0' !!! inac pokracuje slucka dalej !!! ....
}


