#ifndef __PIDREG_H__
#define __PIDREG_H__

#include "arm_math.h"

/* Choose PID parameters */
#define PID_PARAM_KP		159.68	/* Proporcional */
#define PID_PARAM_TI		230.00	/* Integral */
#define PID_PARAM_TD		57.50		/* Derivative */

#define GMA_HLIM (100.0) 				// PID controller upper limit [%]
#define GMA_LLIM (0.0)					// PID controller lower limit [%]

enum {NOR=0, PID=1, ALE=2, TAK=3, PSD=4, DSP=5, ARM=6, FUZ=7};
enum {P_Type=1, PI_Type=2, PD_Type=3, PID_Type=4};
enum {ZIE=1, COH=2, ITA=3};

#define	Numbers_Regulators	4					//Pocet pouzitych regulatorov od REGK0 do REGV1 v RTX51 max. 4.... viac nie !!!

#define RegKSrvMin			0
#define RegKSrvZdvih		12
#define RegKSrvPosun		15
#define	RegKSrvMax			(RegKSrvZdvih*RegKSrvPosun)																									//Natocenie o 180 za 180 sekund, Zdvih=12mm, Posun=15s/mm = 12*15=180s
#define RegKSrvHyster		2
#define RegVSrvMin			0
#define RegVSrvZdvih		12
#define RegVSrvPosun		3
#define	RegVSrvMax			(RegVSrvZdvih*RegVSrvPosun)																									//Natocenie o 180 za 36 sekund, Zdvih=12mm, Posun=3s/mm = 12*3=36s
#define RegVSrvHyster		2

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#define	Tu					115.0										// Tu - cas prietahu tzv. Death-Time vid. J. BALTE: AUTOMATICK RZEN str. 234
#define	Tn					14691.0									// Tn - cas nabehu
#define	dy					180.0										// Rozsah servoventilu, alebo maxima statickej sustavy ...
#define	du					(double_t)RegKSrvZdvih		// Max. hodnota zdvihu servoventilu
#define	ks					(dy/du)									// ks - koeficient prenosu (zosilnenie), alebo statick zosilnenie
#define	a0					(1.0/ks)								// a0 - Cinitel samoregulcie
#define	kr_p				(1.25*(Tn/Tu)*(1.0/ks))	// Kr* - proporcionalna konstanta s rozmerom
#define	Tik					(2.0*Tu)								// Tik - integracna konstanta
#define Tdk					(0.5*Tu)								// Tdk - derivacna konstanta
#define	ddyr				(du/kr_p)								// ddyr - hodnota na max. zdvih
#define	pp_p				((ddyr/dy)*100.0)				// pp_k - psmo proporcionality
#define	Kr_k				((1.0/pp_p)*100.0)			// Kr_k - proporcionalna konstanta bez rozmeru ktoru potrebujem ...
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#define	Metoda_PID	ZIE									
#define RKkc				Kr_k							//156.2
#define RKti				Tik								//230.0
#define RKtd				Tdk								//57.5
#define	RKts				0.100							//ts=vzorkovanie regulatora, v tasku nesmie prekrocit 254ms t.j. 0.254 !!!
#define	RKk_lpf			0.1*RKtd					//filtracia D-zlozky
#define RVkc				Kr_k
#define RVti				Tik
#define RVtd				Tdk
#define	RVts				0.100							//ts=vzorkovanie regulatora, v tasku nesmie prekrocit 254ms t.j. 0.254 !!!
#define	RVk_lpf			0.1*RVtd					//filtracia D-zlozky
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#define alfak (RKts+2.0*RKk_lpf)
#define alfav (RVts+2.0*RVk_lpf)
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

typedef struct _pid_params
{
	double_t kc; 		// Controller gain for P from Dialog Box
	double_t ti; 		// Time-constant for I action from Dialog Box
	double_t td; 		// Time-constant for D action from Dialog Box
	double_t ts; 		// Sample time [sec.] from Dialog Box
	double_t k_lpf; 	// Time constant [sec.] for LPF filter
	double_t k0; 		// k0 value for PID controller
	double_t k1; 		// k1 value for PID controller
	double_t k2; 		// k2 value for PID controller
	double_t k3; 		// k3 value for PID controller
	double_t lpf1;		// value for LPF filter
	double_t lpf2; 	// value for LPF filter
	uint8_t os_wait_ts_ticks; // ticks for timer
	int16_t pid_model;// PID Controller type [0..3]
	double_t pp; 		// debug
	double_t pi; 		// debug
	double_t pd; 		// debug
	double_t Temp;		// Temporary value for WinDup
	double_t Soll;		// Ziadana hodnota teploty ...
	int16_t SetPosition,ActualPosition,dPosition;
	int16_t Servo_Min,Servo_Max,Pump,Hysterezia;
	int16_t Tkomfort,Tutlm,deltaT,Tmin,Tmax,Tlimit;
	int16_t Metoda;					//Metoda vypoctu konstant regulatora ...
	uint8_t Ekviterm;			//Ekvitermicka - vykurovacia krivka ...
	uint8_t Ekviterm_old;	//Ekvitermicka - vykurovacia krivka ...
}
pid_params; // struct pid_params

typedef struct	_var_params
{
	double_t ek_1; 	// e[k-1] = SP[k-1] - PV[k-1] = Tset_hlt[k-1] - Thlt[k-1]
	double_t ek_2; 	// e[k-2] = SP[k-2] - PV[k-2] = Tset_hlt[k-2] - Thlt[k-2]
	double_t xk_1; 	// PV[k-1] = Thlt[k-1]
	double_t xk_2; 	// PV[k-2] = Thlt[k-1]
	double_t yk_1; 	// y[k-1] = Gamma[k-1]
	double_t yk_2; 	// y[k-2] = Gamma[k-1]
	double_t lpf_1;	// lpf[k-1] = LPF output[k-1]
	double_t lpf_2;	// lpf[k-2] = LPF output[k-2]
}
var_params;	// struct var_params

#endif


