#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pidreg.h"
#include "arm_math.h"

/*
Purpose : This file contains the main body of the PSD float.
For design details, please read the Word document
"PSD Controller Calculus".
In the GUI, the following parameters can be changed:
kc: The float32_t gain
Ti: Time-constant for the Integral Gain
Td: Time-constant for the Derivative Gain
Ts: The sample period [seconds]
*/


//--------------------
// Function Prototypes
//--------------------

void init_arm_k0(pid_params *p);
void init_arm_v0(pid_params *p);
void reg_arm_k0(double_t xk, double_t *yk, pid_params *p, var_params *v);
void reg_arm_v0(double_t xk, double_t *yk, pid_params *p, var_params *v);
	
// ARM PID Instance, double_32 format
arm_pid_instance_f32 PSD_k0;
arm_pid_instance_f32 PSD_v0;
extern void arm_psd_init_f32(arm_pid_instance_f32*, int32_t);

void init_arm_k0(pid_params *p)
{
 //////p->os_wait_ts_ticks=(unsigned char)(p->ts*1000);				//BRM (Backward rectangule method) spatna lichobeznikova metoda, + a - su pre kladne scitania v ARM vypocte
 PSD_k0.Kp=(p->kc*(1.0+(p->ts/(2.0*p->ti))+(p->td/p->ts)));		//Konstanta KA prirastkoveho regulatora
 PSD_k0.Ki=(p->kc*(1.0-(p->ts/(2.0*p->ti))+(2*p->td/p->ts)));	//Konstanta KB prirastkoveho regulatora
 PSD_k0.Kd=(p->kc*(p->td/(p->ts)));				     								//Konstanta KC prirastkoveho regulatora 
//Initialize PSD_k0 system, double32_t format
 arm_psd_init_f32(&PSD_k0,1);																	//Inicializacia instancie pre PSD ...
}

void reg_arm_k0(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	float32_t ek;																						// e[k]
	ek = p->Soll - xk; 																	// calculate e[k]
	if (p->pid_model)
		{
		 	*yk= arm_psd_f32(&PSD_k0,ek);										//Y0=Y1+A0*E0-A1*E1+A2*E2;
		}
	else *yk = +0.0;
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
}

void init_arm_v0(pid_params *p)
{
 //////p->os_wait_ts_ticks=(unsigned char)(p->ts*1000);				//BRM (Backward rectangule method) spatna lichobeznikova metoda, + a - su pre kladne scitania v ARM vypocte
 PSD_v0.Kp=(p->kc*(1.0+(p->ts/(2.0*p->ti))+(p->td/p->ts)));		//Konstanta KA prirastkoveho regulatora
 PSD_v0.Ki=(p->kc*(1.0-(p->ts/(2.0*p->ti))+(2*p->td/p->ts)));	//Konstanta KB prirastkoveho regulatora
 PSD_v0.Kd=(p->kc*(p->td/(p->ts)));				     								//Konstanta KC prirastkoveho regulatora 
//Initialize PSD_k0 system, double32_t format
 arm_psd_init_f32(&PSD_v0,1);																	//Inicializacia instancie pre PSD ...
}

void reg_arm_v0(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	float32_t ek;																						// e[k]
	ek = p->Soll - xk; 																	// calculate e[k]
	if (p->pid_model)
		{
		 	*yk= arm_psd_f32(&PSD_v0,ek);										//Y0=Y1+A0*E0-A1*E1+A2*E2;
		}
	else *yk = +0.0;
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
}

