#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pidreg.h"
#include "arm_math.h"

/*
Purpose : This file contains the main body of the PSD float.
For design details, please read the Word document
"PSD Controller Calculus".
In the GUI, the following parameters can be changed:
kc: The float32_t gain
Ti: Time-constant for the Integral Gain
Td: Time-constant for the Derivative Gain
Ts: The sample period [seconds]
*/


//--------------------
// Function Prototypes
//--------------------

void init_psd_k0(pid_params *p);
void reg_psd_k0(double_t xk, double_t *yk, pid_params *p, var_params *v);
void init_psd_v0(pid_params *p);
void reg_psd_v0(double_t xk, double_t *yk, pid_params *p, var_params *v);

void init_psd_k1(pid_params *p);
void reg_psd_k1(double_t xk, double_t *yk, pid_params *p, var_params *v);
void init_psd_v1(pid_params *p);
void reg_psd_v1(double_t xk, double_t *yk, pid_params *p, var_params *v);

void init_psd_k0(pid_params *p)
{
 //p->os_wait_ts_ticks=(unsigned char)(p->ts*1000);
 p->k0=(p->kc*(1.0+(p->ts/(2.0*p->ti))+(p->td/p->ts)));			//Konstanta KA prirastkoveho regulatora
 p->k1=(p->kc*(1.0-(p->ts/(2.0*p->ti))+(2.0*p->td/p->ts)));	//Konstanta KB prirastkoveho regulatora
 p->k2=(p->kc*(p->td/(p->ts)));				     									//Konstanta KC prirastkoveho regulatora 
}

void reg_psd_k0(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	double_t ek;																						// e[k]
	ek = p->Soll - xk; 																		// calculate e[k]
	if (p->pid_model)
		{
		 *yk=v->yk_1+p->k0*ek-p->k1*v->ek_1+p->k2*v->ek_2;//y0=y1+k0*e0-k1*e1+k2*e2;
		}
	else *yk = +0.0;
	v->ek_2 = v->ek_1;																	// e[k-2] = e[k-1]
	v->ek_1 = ek; 																			// e[k-1] = e[k]
	//v->yk_2 = v->yk_1; 																	// y[k-2] = y[k-1]
	v->yk_1 = *yk; 																			// y[k-1] = y[k]
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
}

void init_psd_v0(pid_params *p)
{
 //p->os_wait_ts_ticks=(unsigned char)(p->ts*1000);
 p->k0=(p->kc*(1.0+(p->ts/(2.0*p->ti))+(p->td/p->ts)));			//Konstanta KA prirastkoveho regulatora
 p->k1=(p->kc*(1.0-(p->ts/(2.0*p->ti))+(2.0*p->td/p->ts)));	//Konstanta KB prirastkoveho regulatora
 p->k2=(p->kc*(p->td/(p->ts)));				     									//Konstanta KC prirastkoveho regulatora 
}

void reg_psd_v0(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	double_t ek;																						// e[k]
	ek = p->Soll - xk; 																		// calculate e[k]
	if (p->pid_model)
		{
		 *yk=v->yk_1+p->k0*ek-p->k1*v->ek_1+p->k2*v->ek_2;//y0=y1+k0*e0-k1*e1+k2*e2;
		}
	else *yk = +0.0;
	v->ek_2 = v->ek_1;																	// e[k-2] = e[k-1]
	v->ek_1 = ek; 																			// e[k-1] = e[k]
	//v->yk_2 = v->yk_1; 																	// y[k-2] = y[k-1]
	v->yk_1 = *yk; 																			// y[k-1] = y[k]
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
}

//*******************************************************************************************************************

void init_psd_k1(pid_params *p)
{
 //p->os_wait_ts_ticks=(unsigned char)(p->ts*1000);
 p->k0=(p->kc*(1.0+(p->ts/(2.0*p->ti))+(p->td/p->ts)));			//Konstanta KA prirastkoveho regulatora
 p->k1=(p->kc*(1.0-(p->ts/(2.0*p->ti))+(2.0*p->td/p->ts)));	//Konstanta KB prirastkoveho regulatora
 p->k2=(p->kc*(p->td/(p->ts)));				     									//Konstanta KC prirastkoveho regulatora 
}

void reg_psd_k1(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	double_t ek;																						// e[k]
	ek = p->Soll - xk; 																		// calculate e[k]
	if (p->pid_model)
		{
		 *yk=v->yk_1+p->k0*ek-p->k1*v->ek_1+p->k2*v->ek_2;//y0=y1+k0*e0-k1*e1+k2*e2;
		}
	else *yk = +0.0;
	v->ek_2 = v->ek_1;																	// e[k-2] = e[k-1]
	v->ek_1 = ek; 																			// e[k-1] = e[k]
	//v->yk_2 = v->yk_1; 																	// y[k-2] = y[k-1]
	v->yk_1 = *yk; 																			// y[k-1] = y[k]
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
}

void init_psd_v1(pid_params *p)
{
 //p->os_wait_ts_ticks=(unsigned char)(p->ts*1000);
 p->k0=(p->kc*(1.0+(p->ts/(2.0*p->ti))+(p->td/p->ts)));			//Konstanta KA prirastkoveho regulatora
 p->k1=(p->kc*(1.0-(p->ts/(2.0*p->ti))+(2.0*p->td/p->ts)));	//Konstanta KB prirastkoveho regulatora
 p->k2=(p->kc*(p->td/(p->ts)));				     									//Konstanta KC prirastkoveho regulatora 
}

void reg_psd_v1(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	double_t ek;																						// e[k]
	ek = p->Soll - xk; 																		// calculate e[k]
	if (p->pid_model)
		{
		 *yk=v->yk_1+p->k0*ek-p->k1*v->ek_1+p->k2*v->ek_2;//y0=y1+k0*e0-k1*e1+k2*e2;
		}
	else *yk = +0.0;
	v->ek_2 = v->ek_1;																	// e[k-2] = e[k-1]
	v->ek_1 = ek; 																			// e[k-1] = e[k]
	//v->yk_2 = v->yk_1; 																	// y[k-2] = y[k-1]
	v->yk_1 = *yk; 																			// y[k-1] = y[k]
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
}

