/**
 *	Keil project for ARM PID example
 *
 *	Before you start, select your target, on the right of the "Load" button
 *
 *	@author		Tilen Majerle
 *	@email		tilen@majerle.eu
 *	@website	http://stm32f4-discovery.com
 *	@ide			Keil uVision 5
 *	@packs		STM32F4xx Keil packs version 2.2.0 or greater required
 *	@stdperiph	STM32F4xx Standard peripheral drivers version 1.4.0 or greater required
 *
 *	Additional defines in "Options for Target" > "C/C++" > "Defines"
 *		- ARM_MATH_CM4
 *		- __FPU_PRESENT = 1
 */
/* Include core modules */
#include "stm32f4xx.h"
/* Include my libraries here */
#include "defines.h"
#include "tm_stm32f4_delay.h"
#include "tm_stm32f4_onewire.h"
#include "tm_stm32f4_ds18b20.h"
#include "tm_stm32f4_disco.h"
#include "tm_stm32f4_usart.h"
#include "tm_stm32f4_spi.h"
#include "tm_stm32f4_spi_dma.h"
#include "tm_stm32f4_pwm.h"
#include "tm_stm32f4_ili9341.h"
#include "tm_stm32f4_rotary_encoder.h"
#include "tm_stm32f4_adc.h"
#include "tm_stm32f4_dac.h"
#include "tm_stm32f4_rtc.h"
#include "tm_stm32f4_i2c.h"

/* Include ARM math */
#include "arm_math.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "pidreg.h"

int h,m,s,day,year;
char month[sizeof("Jan")];
char *Month[]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
char Date[]=__DATE__;
char Time[]=__TIME__;
uint8_t Transmit[15], Receive[15],i;

char buf[150];
//**************************************************************************************************************************************************************************************************************************************************
#define	REGK0					0
#define	REGV0					1
#define	REGK1					2
#define	REGV1					3
//**************************************************************************************************************************************************************************************************************************************************
double xk[Numbers_Regulators],yk[Numbers_Regulators];
pid_params par[Numbers_Regulators],*ptr_par[Numbers_Regulators];
var_params var[Numbers_Regulators],*ptr_var[Numbers_Regulators];
//**************************************************************************************************************************************************************************************************************************************************
extern void Vypocet_PID_Konstant(pid_params*,uint8_t,uint8_t,double,double,double);
//**************************************************************************************************************************************************************************************************************************************************
extern pid_params pid[Numbers_Regulators];										//PID+ALE+TAK+PSD+DSP with LPF
//**************************************************************************************************************************************************************************************************************************************************
extern void init_pid_k0(pid_params*);
extern void init_pid_v0(pid_params*);
extern void init_ale_k0(pid_params*);
extern void init_ale_v0(pid_params*);
extern void init_tak_k0(pid_params*);
extern void init_tak_v0(pid_params*);
extern void init_psd_k0(pid_params*);
extern void init_psd_v0(pid_params*);
extern void init_dsp_k0(pid_params*);
extern void init_dsp_v0(pid_params*);
extern void init_arm_k0(pid_params*);
extern void init_arm_v0(pid_params*);
//**************************************************************************************************************************************************************************************************************************************************
extern void reg_pid_k0(double,double*,pid_params*,var_params*);	//PID with LPF
extern void reg_pid_v0(double,double*,pid_params*,var_params*);	//PID with LPF
extern void reg_ale_k0(double,double*,pid_params*,var_params*);	//Allen Bradley with LPF
extern void reg_ale_v0(double,double*,pid_params*,var_params*);	//Allen Bradley with LPF
extern void reg_tak_k0(double,double*,pid_params*,var_params*);	//Takahashi without LPF
extern void reg_tak_v0(double,double*,pid_params*,var_params*);	//Takahashi without LPF
extern void reg_psd_k0(double,double*,pid_params*,var_params*);	//Velocity PSD without LPF
extern void reg_psd_v0(double,double*,pid_params*,var_params*);	//Velocity PSD without LPF
extern void reg_dsp_k0(double,double*,pid_params*,var_params*);	//CMSIS DSP PID
extern void reg_dsp_v0(double,double*,pid_params*,var_params*);	//CMSIS DSP PID
extern void reg_arm_k0(double,double*,pid_params*,var_params*);	//Velocity PSD without LPF CMSIS DSP ARM
extern void reg_arm_v0(double,double*,pid_params*,var_params*);	//Velocity PSD without LPF CMSIS DSP ARM
//**************************************************************************************************************************************************************************************************************************************************

TM_RTC_Time_t datatime;

void EJ_DAC_Setting(TM_DAC_Channel_t DACx, float32_t duty_percent)
{
 uint16_t DAC_Value;
 DAC_Value=(uint16_t)duty_percent*(4096.0/100.0);
 TM_DAC_SetValue(DACx, DAC_Value);
}

/* Timer data for PWM */
TM_PWM_TIM_t TIM_Data;
/* Duty cycle for PWM */
float32_t duty = 0.5;

/* Custom request handler function */
/* Called on wakeup interrupt */
void TM_RTC_RequestHandler() 
{
	/* Get time */
	TM_RTC_GetDateTime(&datatime, TM_RTC_Format_BIN);
	/* Format time */
	sprintf(buf, "%02d.%02d.%04d %02d:%02d:%02d  Unix: %u\n",datatime.date,datatime.month,datatime.year + 2000,datatime.hours,datatime.minutes,datatime.seconds,datatime.unix);
	/* Send to USART */
	//TM_USART_Puts(UART8, buf);
	/* Toggle LED */
	TM_DISCO_LedToggle(LED_GREEN | LED_ORANGE);
	//TM_DISCO_SetLed(LED_GREEN,0);
	//TM_DISCO_SetLed(LED_ORANGE,0);
	//Delay(25000);
} 

int main(void) 
{
/* Initialize system */
	SystemInit();

	/* Initialize delay */
	TM_DELAY_Init();
	
	/* Initialize Leds */
	TM_DISCO_LedInit();

	/* Initialize button on board K0->PE4 K1->PE3 */
	TM_DISCO_ButtonInit();
	
	/* Initialize DAC channel 1, pin PA4 */
	TM_DAC_Init(TM_DAC1);
	/* Initialize DAC channel 2, pin PA5 */
	TM_DAC_Init(TM_DAC2);
	
	/* Initialize ADC1 on channel 0, this is pin PA0 */
	TM_ADC_Init(ADC1, ADC_Channel_0);
	/* Initialize ADC1 on channel 1, this is pin PA1 */
	TM_ADC_Init(ADC1, ADC_Channel_1);
	
	/* Initialize RTC with internal 32.768kHz clock */
	/* It's very accurate XTAL = PC14 & PC15*/
	#ifdef ExternalSource
	//TM_RTC_Init(TM_RTC_ClockSource_External);
	if(!TM_RTC_Init(TM_RTC_ClockSource_External))
	#else
	/* Initialize RTC with internal 32.768kHz clock */
	/* It's not very accurate */
	//TM_RTC_Init(TM_RTC_ClockSource_Internal);
	if(!TM_RTC_Init(TM_RTC_ClockSource_Internal))
	#endif
	{
		/* RTC was first time initialized */
		/* Do your stuff here */
		/* eg. set default time */
		sscanf(Time,"%02d:%02d:%02d",&h,&m,&s);
		sscanf(Date,"%s %02d %4d",month,&day,&year);
		i=0;
		while(memcmp(month,Month[i++],sizeof("Jan"))!=0) Delay(10);	
		datatime.hours = (uint8_t)h;
		datatime.minutes = (uint8_t)m;
		datatime.seconds = (uint8_t)s;
		datatime.year = (uint8_t)(year-2000);
		datatime.date = (uint8_t)day;
		datatime.month=i;
		datatime.day = 1;
		/* Set new time */
		TM_RTC_SetDateTime(&datatime, TM_RTC_Format_BIN);
	}

	 /* Set wakeup interrupt every 0.125 second */
	 //TM_RTC_Interrupts(TM_RTC_Int_1s);
	 //TM_RTC_Interrupts(TM_RTC_Int_500ms);
	 TM_RTC_Interrupts(TM_RTC_Int_125ms);

	 ptr_par[REGK0]=((pid_params*)&par[REGK0]);															//Nacitam adresu objektu a dosadim do premennej typu ukazovatel aby som mal kompatibilitu s predchadzajucim PID2 napisanym v jazyku C51 pre 80C552
	 ptr_var[REGK0]=((var_params*)&var[REGK0]);															//Nacitam adresu objektu a dosadim do premennej typu ukazovatel aby som mal kompatibilitu s predchadzajucim PID2 napisanym v jazyku C51 pre 80C552	 
	 ptr_par[REGV0]=((pid_params*)&par[REGV0]);															//Nacitam adresu objektu a dosadim do premennej typu ukazovatel aby som mal kompatibilitu s predchadzajucim PID2 napisanym v jazyku C51 pre 80C552
	 ptr_var[REGV0]=((var_params*)&var[REGV0]);															//Nacitam adresu objektu a dosadim do premennej typu ukazovatel aby som mal kompatibilitu s predchadzajucim PID2 napisanym v jazyku C51 pre 80C552

	 memset(ptr_par[REGK0],NULL,sizeof(pid_params));												//Pridelim dynamicku pamat pre struktury a inicializujem ju ...
	 memset(ptr_par[REGV0],NULL,sizeof(pid_params));												//Pridelim dynamicku pamat pre struktury a inicializujem ju ...
	 memset(ptr_var[REGK0],NULL,sizeof(var_params));												//Pridelim dynamicku pamat pre struktury a inicializujem ju ...
	 memset(ptr_var[REGV0],NULL,sizeof(var_params));												//Pridelim dynamicku pamat pre struktury a inicializujem ju ...
	 
	 #ifdef SIMULxxx
	 memcpy(ptr_par[REGK0],&pid[REGK0],sizeof(pid_params));									//Nacitaj factory hodnoty z pamati programu ...
	 memcpy(ptr_par[REGV0],&pid[REGV0],sizeof(pid_params));									//Nacitaj factory hodnoty z pamati programu ...
	 #endif

	 ptr_par[REGK0]->Metoda=ZIE;																						//Volba metody vypoctu konstant Kp, Ti, Td regulatora pomocou Ziegler-Nichols, Cohen-Coon, ITA Load ....
	 ptr_par[REGV0]->Metoda=ZIE;

	 ptr_par[REGK0]->pid_model=ARM;																					//Vyber konkretneho typu regulatora DSP-LPF, Allen Bradley-LPF, Takahashi bez filtracie D zlozky - LPF, PSD bez LPF ...
	 ptr_par[REGV0]->pid_model=ARM;																					//Vyber konkretneho typu regulatora DSP-LPF, Allen Bradley-LPF, Takahashi bez filtracie D zlozky - LPF, PSD bez LPF ...  
 
	 switch(ptr_par[REGK0]->Metoda)
	  {
		 default	:
		 case	ZIE	:	Vypocet_PID_Konstant(ptr_par[REGK0],ZIE,PID_Type,+115.0,+14691.0,+0.000);	break;	//Vypocet nepotrebuje _Khlt a _Thlt co sa zistuje komplikovanejsie ...
		 case	COH	:	Vypocet_PID_Konstant(ptr_par[REGK0],COH,PID_Type,+115.0,+14691.0,+1.689);	break;	//Vypocet potrebuje _Khlt a _Thlt regulovanej sustavy, zistenie je obtiaznejsie ...
		 case	ITA	: Vypocet_PID_Konstant(ptr_par[REGK0],ITA,PID_Type,+115.0,+14691.0,+1.689);	break;	//Vypocet potrebuje _Khlt a _Thlt regulovanej sustavy, zistenie je obtiaznejsie ...
		}
	 switch(ptr_par[REGV0]->Metoda)
	  {
		 default	:
		 case	ZIE	:	Vypocet_PID_Konstant(ptr_par[REGV0],ZIE,PID_Type,+115.0,+14691.0,+0.000);	break;	//Vypocet nepotrebuje _Khlt a _Thlt co sa zistuje komplikovanejsie ...
		 case	COH	:	Vypocet_PID_Konstant(ptr_par[REGV0],COH,PID_Type,+115.0,+14691.0,+1.689);	break;	//Vypocet potrebuje _Khlt a _Thlt regulovanej sustavy, zistenie je obtiaznejsie ...
		 case	ITA	: Vypocet_PID_Konstant(ptr_par[REGV0],ITA,PID_Type,+115.0,+14691.0,+1.689);	break;	//Vypocet potrebuje _Khlt a _Thlt regulovanej sustavy, zistenie je obtiaznejsie ...
		}
	 
	 ptr_par[REGK0]->k_lpf=0.1*ptr_par[REGK0]->td;													//Musia sa naplnit aj ine parametre, lebo to nespravne funguje ...
	 ptr_par[REGV0]->k_lpf=0.1*ptr_par[REGV0]->td;													//Musia sa naplnit aj ine parametre, lebo to nespravne funguje ...

	 ptr_par[REGK0]->Soll=20.0;																							//Nacitam ziadanu hodnotu ...
	 ptr_par[REGV0]->Soll=50.0;																							//Nacitam ziadanu hodnotu ...

	 ptr_par[REGK0]->Ekviterm=13;																						//Nastavim vykurovaciu krivku, musi byt nastavena pred spustenim vsetkych ostatnych taskov, inac to nefunguje ...
	 ptr_par[REGV0]->Ekviterm=13;																						//Nastavim vykurovaciu krivku, musi byt nastavena pred spustenim vsetkych ostatnych taskov, inac to nefunguje ...
	 
 	 ptr_par[REGK0]->ts=0.1;																								//Musia sa naplnit aj ine parametre, lebo to nespravne funguje ...
	 ptr_par[REGV0]->ts=0.1;																								//Musia sa naplnit aj ine parametre, lebo to nespravne funguje ...

	 switch(ptr_par[REGK0]->pid_model)
	 {
		default		:
		case	PID	:	init_pid_k0(ptr_par[REGK0]);	break;
		case	ALE	:	init_ale_k0(ptr_par[REGK0]);	break;
		case	TAK	:	init_tak_k0(ptr_par[REGK0]);	break;
		case	PSD	:	init_psd_k0(ptr_par[REGK0]);	break;
		case	DSP	:	init_dsp_k0(ptr_par[REGK0]);	break;											//Kedze nema obmedzenu integracnu/sumacnu zlozku, moze po dostatocnom "naintegrovani" hodnot pomalsie reagovat na vystupe regulatora
		case	ARM	:	init_arm_k0(ptr_par[REGK0]);	break;											//Kedze nema obmedzenu integracnu/sumacnu zlozku, moze po dostatocnom "naintegrovani" hodnot pomalsie reagovat na vystupe regulatora
	 }
	 switch(ptr_par[REGV0]->pid_model)
	 {
		default		:
		case	PID	:	init_pid_v0(ptr_par[REGV0]);	break;
		case	ALE	:	init_ale_v0(ptr_par[REGV0]);	break;
		case	TAK	:	init_tak_v0(ptr_par[REGV0]);	break;
		case	PSD	:	init_psd_v0(ptr_par[REGV0]);	break;
		case	DSP	:	init_dsp_v0(ptr_par[REGV0]);	break;											//Kedze nema obmedzenu integracnu/sumacnu zlozku, moze po dostatocnom "naintegrovani" hodnot pomalsie reagovat na vystupe regulatora
		case	ARM	:	init_arm_v0(ptr_par[REGV0]);	break;											//Kedze nema obmedzenu integracnu/sumacnu zlozku, moze po dostatocnom "naintegrovani" hodnot pomalsie reagovat na vystupe regulatora
	 }
	 
	 while(1) 
		{
			if(TM_DISCO_ButtonPressed_K0()!=1) TM_DISCO_SetLed(LED_ORANGE,1);	// else TM_DISCO_SetLed(LED_ORANGE,0);
			if(TM_DISCO_ButtonPressed_K1()!=1) TM_DISCO_SetLed(LED_GREEN,1);	// else TM_DISCO_SetLed(LED_GREEN,0);
		
			xk[REGK0]=(TM_ADC_Read(ADC1,ADC_Channel_0)*(100.0/4096.0));														//Nacitam skutocnu hodnotu nameranu na AD prevodniku ...

			switch(ptr_par[REGK0]->pid_model)
			 {
				default		:																																					//ak nie je zvoleny ziaden regulator, tak musi byt spusteny defaultne vypnuty PID a to je tu s parametrom par[REGK0]->pid_model=0
				case	PID	:	reg_pid_k0(xk[REGK0],&yk[REGK0],ptr_par[REGK0],ptr_var[REGK0]); break;	//Doplnene o vypocet na zaklade vykurovacej krivky - Heating Curve
				case	ALE	:	reg_ale_k0(xk[REGK0],&yk[REGK0],ptr_par[REGK0],ptr_var[REGK0]); break;	//Doplnene o vypocet na zaklade vykurovacej krivky - Heating Curve
				case	TAK	:	reg_tak_k0(xk[REGK0],&yk[REGK0],ptr_par[REGK0],ptr_var[REGK0]); break;	//Doplnene o vypocet na zaklade vykurovacej krivky - Heating Curve
				case	PSD	:	reg_psd_k0(xk[REGK0],&yk[REGK0],ptr_par[REGK0],ptr_var[REGK0]); break;	//Doplnene o vypocet na zaklade vykurovacej krivky - Heating Curve
				case	DSP	:	reg_dsp_k0(xk[REGK0],&yk[REGK0],ptr_par[REGK0],ptr_var[REGK0]); break;	//Doplnene o vypocet na zaklade vykurovacej krivky - Heating Curve
				case	ARM	:	reg_arm_k0(xk[REGK0],&yk[REGK0],ptr_par[REGK0],ptr_var[REGK0]); break;	//Doplnene o vypocet na zaklade vykurovacej krivky - Heating Curve
			 }
			
			EJ_DAC_Setting(TM_DAC1,yk[REGK0]);																										//Nastavim akcny zasah do regulovanej sustavy cez DAC prevodnik ....
			
			xk[REGV0]=(TM_ADC_Read(ADC1,ADC_Channel_1)*(100.0/4096.0));														//Nacitam skutocnu hodnotu nameranu na AD prevodniku ...
			
			switch(ptr_par[REGV0]->pid_model)
			 {
				default		:																																					//ak nie je zvoleny ziaden regulator, tak musi byt spusteny defaultne vypnuty PID a to je tu s parametrom par[REGK0]->pid_model=0
				case	PID	:	reg_pid_v0(xk[REGV0],&yk[REGV0],ptr_par[REGV0],ptr_var[REGV0]); break;	//Doplnene o vypocet na zaklade vykurovacej krivky - Heating Curve
				case	ALE	:	reg_ale_v0(xk[REGV0],&yk[REGV0],ptr_par[REGV0],ptr_var[REGV0]); break;	//Doplnene o vypocet na zaklade vykurovacej krivky - Heating Curve
				case	TAK	:	reg_tak_v0(xk[REGV0],&yk[REGV0],ptr_par[REGV0],ptr_var[REGV0]); break;	//Doplnene o vypocet na zaklade vykurovacej krivky - Heating Curve
				case	PSD	:	reg_psd_v0(xk[REGV0],&yk[REGV0],ptr_par[REGV0],ptr_var[REGV0]); break;	//Doplnene o vypocet na zaklade vykurovacej krivky - Heating Curve
				case	DSP	:	reg_dsp_v0(xk[REGV0],&yk[REGV0],ptr_par[REGV0],ptr_var[REGV0]); break;	//Doplnene o vypocet na zaklade vykurovacej krivky - Heating Curve
				case	ARM	:	reg_arm_v0(xk[REGK0],&yk[REGK0],ptr_par[REGK0],ptr_var[REGK0]); break;	//Doplnene o vypocet na zaklade vykurovacej krivky - Heating Curve
			 }
				 
			EJ_DAC_Setting(TM_DAC2,yk[REGV0]);																										//Nastavim akcny zasah do regulovanej sustavy cez DAC prevodnik ....
		
			Delay(100000);
	}
}







