/*----------------------------------------------------------------------------
 * CMSIS-RTOS 'main' function template
 *---------------------------------------------------------------------------*/

#include "RTE_Components.h"
#include  CMSIS_device_header
#include "cmsis_os2.h"
#include "rtx_os.h"

#ifdef RTE_Compiler_EventRecorder
#include "EventRecorder.h"
#endif

#include "tm_stm32f4_delay.h"
#include "tm_stm32f4_disco.h"
#include "tm_stm32f4_adc.h"
#include "tm_stm32f4_dac.h"
#include "tm_stm32f4_rtc.h"
#include "tm_stm32f4_i2c.h"
#include "tm_stm32f4_usart.h"
#include "tm_stm32f4_nrf24l01.h"
#include "tm_stm32f4_dht11.h"
#include "defines.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define	K_TMO	100
#define	REGK0	0x0001
#define	REGV0	0x0002

typedef struct Time_
{
 int h,m,s,day,year;
}
Times;

Times Cas;

int i;
char month[sizeof("Jan")];
char *Month[]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
char Date[]=__DATE__,Time[]=__TIME__;
char buf[150];

TM_DHT11_Data_t data;
char str[50];
	
TM_RTC_Time_t datatime;

typedef struct msg_s 
{
  uint8_t cmd;
  uint32_t len;
  uint32_t data;
}
msg_t;

uint32_t param0,param1,param2,param3,param4,param5,param6;

__attribute__((section(".bss.os.msgqueue.cb")))
osMessageQueueId_t msgQueue;
__attribute__((section(".bss.os.evflags.cb")))
osEventFlagsId_t evt_id,sig_id,clk_id;
__attribute__((section(".bss.os.thread.cb"))) 
osThreadId_t th0,th1,th2,th3,th4,th5,th6;

// Reserve two areas for the stacks of worker thread 1
// uint64_t makes sure the memory alignment is 8
//__attribute__((section(".bss.os.stack.cb")))
uint64_t admin_app_thread_stk[64];
uint64_t reg_k0_thread_stk[64];
uint64_t reg_v0_thread_stk[64];
uint64_t msg_app_thread_stk[64];
uint64_t flag_app_thread_stk[64];
uint64_t sig_app_thread_stk[64];
uint64_t clk_app_thread_stk[64];

// Define objects that are ally allocated for thread
__attribute__((section(".bss.os.thread.cb")))
osRtxThread_t admin_app_thread_tcb;
osRtxThread_t reg_k0_thread_tcb;
osRtxThread_t reg_v0_thread_tcb;
osRtxThread_t msg_app_thread_tcb;
osRtxThread_t flag_app_thread_tcb;
osRtxThread_t sig_app_thread_tcb;
osRtxThread_t clk_app_thread_tcb;

int16_t Counter=0;

/* Called on wakeup interrupt */
void TM_RTC_RequestHandler() 
{
	switch(Counter++)
	 {
		default	:	osEventFlagsSet(clk_id,0x01);
							break;
	  case	3	:	osEventFlagsSet(clk_id,0x02);
							Counter=0;
							break;
	  case	5	:	osEventFlagsSet(clk_id,0x04);
							Counter=0;
							break;
	 }
}

// Define the attributes which are used for thread creation
// Optional const saves RAM memory and includes the values in periodic ROM tests 

const osThreadAttr_t admin_app_attr=
{
  .name="admin_app_attr",
	.attr_bits=osThreadDetached,
	.cb_mem=&admin_app_thread_tcb,
	.cb_size=sizeof(admin_app_thread_tcb),
	.stack_mem=&admin_app_thread_stk[0],
	.stack_size=sizeof(admin_app_thread_stk),
	.priority=osPriorityNormal,
	.tz_module=0U,
	.reserved=0U
};

const osThreadAttr_t reg_k0_attr=
{
  .name="regulator_k0_attr",
  .attr_bits=osThreadDetached,
  .cb_mem=&reg_k0_thread_tcb,
  .cb_size=sizeof(reg_k0_thread_tcb),
  .stack_mem=&reg_k0_thread_stk[0],
  .stack_size=sizeof(reg_k0_thread_stk),
  .priority=osPriorityNormal,
	.tz_module=0U,
	.reserved=0U
};

const osThreadAttr_t reg_v0_attr=
{
  .name="regulator_v0_attr",
  .attr_bits=osThreadDetached,
  .cb_mem=&reg_v0_thread_tcb,
  .cb_size=sizeof(reg_v0_thread_tcb),
  .stack_mem=&reg_v0_thread_stk[0],
  .stack_size=sizeof(reg_v0_thread_stk),
  .priority=osPriorityNormal,
	.tz_module=0U,
	.reserved=0U
};

const osThreadAttr_t msg_app_attr=
{
  .name="message_app_attr",
  .attr_bits=osThreadDetached,
  .cb_mem=&msg_app_thread_tcb,
  .cb_size=sizeof(msg_app_thread_tcb),
  .stack_mem=&msg_app_thread_stk[0],
  .stack_size=sizeof(msg_app_thread_stk),
  .priority=osPriorityNormal,
	.tz_module=0U,
	.reserved=0U
};

const osThreadAttr_t flag_app_attr=
{
  .name="flags_app_attr",
  .attr_bits=osThreadDetached,
  .cb_mem=&flag_app_thread_tcb,
  .cb_size=sizeof(flag_app_thread_tcb),
  .stack_mem=&flag_app_thread_stk[0],
  .stack_size=sizeof(flag_app_thread_stk),
  .priority=osPriorityNormal,
	.tz_module=0U,
	.reserved=0U
};

const osThreadAttr_t sig_app_attr=
{
  .name="signal_app_attr",
  .attr_bits=osThreadDetached,
  .cb_mem=&sig_app_thread_tcb,
  .cb_size=sizeof(sig_app_thread_tcb),
  .stack_mem=&sig_app_thread_stk[0],
  .stack_size=sizeof(sig_app_thread_stk),
  .priority=osPriorityNormal,
	.tz_module=0U,
	.reserved=0U
};

const osThreadAttr_t clk_app_attr=
{
  .name="clock_app_attr",
  .attr_bits=osThreadDetached,
  .cb_mem=&clk_app_thread_tcb,
  .cb_size=sizeof(clk_app_thread_tcb),
  .stack_mem=&clk_app_thread_stk[0],
  .stack_size=sizeof(clk_app_thread_stk),
  .priority=osPriorityHigh,
	.tz_module=0U,
	.reserved=0U
};

/*----------------------------------------------------------------------------
 * Application main thread
 *---------------------------------------------------------------------------*/
__NO_RETURN void app_main(void *argument) 
{
	(void)argument;
	//osStatus_t status;
  uint32_t cnt=0UL;
  msg_t msg;
  for(;;)
	{
	 osDelay(K_TMO*5);
	 msg.data=++cnt;
	 msg.len=sizeof(msg);
	 switch((int16_t)osMessageQueuePut(msgQueue,&msg,osPriorityNormal,osWaitForever))
	  {
		 case	osOK							:	cnt++;	break;
		 case	osErrorTimeout		:	break;
		 case	osErrorResource		:	break;
		 case	osErrorParameter	:	break;
		}
	 if((TM_DISCO_ButtonPressed_K0()!=1)&&(TM_DISCO_ButtonPressed_K1()!=1)) TM_DISCO_SetLed(LED_ORANGE|LED_GREEN,1);	// else TM_DISCO_SetLed(LED_ORANGE,0);
	 //TM_DISCO_LedToggle(LED_GREEN | LED_ORANGE);
	}
}

__NO_RETURN void reg_k0(void *argument) 
{
	(void)argument;
  for(;;)
	{
	 osDelay(K_TMO);
	 //osEventFlagsSet(sig_id,REGK0);
	 //osEventFlagsSet(evt_id,REGK0);
	 //osThreadFlagsSet(th4,0x0001);            													// set signal to thread 'thread'
 	 if(TM_DISCO_ButtonPressed_K0()!=1) TM_DISCO_SetLed(LED_ORANGE,1);		// else TM_DISCO_SetLed(LED_ORANGE,0);
	}
}
 
__NO_RETURN void reg_v0(void *argument) 
{
 (void)argument;
  for(;;)
	{
	 osDelay(K_TMO);
	 //osEventFlagsSet(sig_id,REGV0);
	 //osEventFlagsSet(evt_id,REGV0);
	 //osThreadFlagsSet(th4,0x0001);            													// set signal to thread 'thread'
	 if(TM_DISCO_ButtonPressed_K1()!=1) TM_DISCO_SetLed(LED_GREEN,1);			// else TM_DISCO_SetLed(LED_GREEN,0);
	}
}

__NO_RETURN void msg_app(void *argument)
{
 (void)argument;
 msg_t msg;
 // Defer message processing
 osDelay(osMessageQueueGetSpace(msgQueue)*100U);
 for(;;)
  {
    // Wait forever until a message could be received
    //status=osMessageQueueGet(msgQueue,&msg,NULL,osWaitForever);
		switch((uint32_t)osMessageQueueGet(msgQueue,&msg,NULL,osWaitForever))
		 {
			case	osOK	:		switch(msg.data)
											 {
											  case	0	:	msg.data--;	break;
												default	:	msg.data++;	break;
											 }
											//osEventFlagsSet(sig_id,4);
											//osThreadFlagsSet(th4,0x0001);            													// set signal to thread 'thread'
											//TM_DISCO_LedToggle(LED_GREEN | LED_ORANGE);
										break;
		  default			:	msg.data=0;
										break;
		 }
	}
}

__NO_RETURN void flag_app(void *argument) 
{
 (void)argument;
	TM_DISCO_SetLed(LED_GREEN,1);
	TM_DISCO_SetLed(LED_ORANGE,0);
  for(;;)
	{
	 switch(osEventFlagsWait(evt_id,REGK0|REGV0,osFlagsWaitAny,osWaitForever))
	  {
		 case	REGK0									:	/*TM_DISCO_LedToggle(LED_ORANGE);*/	break;
		 case	REGV0									:	/*TM_DISCO_LedToggle(LED_GREEN);*/	break;
		 case	osFlagsErrorISR				:	
		 case	osFlagsErrorUnknown		:	
		 case	osFlagsErrorParameter	:	
		 case	osFlagsErrorResource	:	
		 case	osFlagsErrorTimeout		:	osEventFlagsClear(evt_id,osEventFlagsGet(evt_id));
																	break;
		 }
	}
}

__NO_RETURN void sig_app(void *argument) 
{
 (void)argument;
	TM_DISCO_SetLed(LED_GREEN,1);
	TM_DISCO_SetLed(LED_ORANGE,0);
  for(;;)
	{
	 switch(osEventFlagsWait(sig_id,REGK0|REGV0,osFlagsWaitAny,K_TMO*2))
	  {
		 case	REGK0									:	TM_DISCO_LedToggle(LED_GREEN);	osDelay(K_TMO);	osEventFlagsSet(sig_id,REGV0);	break;
		 case	REGV0									:	TM_DISCO_LedToggle(LED_ORANGE);	osDelay(K_TMO);	osEventFlagsSet(sig_id,REGK0);	break;
		 case	osFlagsErrorISR				:	
		 case	osFlagsErrorUnknown		:	
		 case	osFlagsErrorParameter	:	
		 case	osFlagsErrorResource	:	
		 case	osFlagsErrorTimeout		:	osEventFlagsClear(sig_id,osEventFlagsGet(sig_id));
																	osEventFlagsSet(sig_id,REGK0);
																	break;
		}
	}
}

//int res;

__NO_RETURN void clk_app(void *argument) 
{
	(void)argument;
  for(;;)
	{
	 switch((uint32_t)osEventFlagsWait(clk_id,0x01|0x02|0x04,osFlagsWaitAny,K_TMO*20))
	  {
		 case	0x01									:	TM_RTC_GetDateTime(&datatime,TM_RTC_Format_BIN);
																	sprintf(buf,"%02d.%02d.%04d %02d:%02d:%02d  Unix: %u\n\r",datatime.date,datatime.month,datatime.year + 2000,datatime.hours,datatime.minutes,datatime.seconds,datatime.unix);
																	//if(TM_USART_BufferEmpty(USART1)) TM_USART_Puts(USART1,buf);
																	break;
		 case	0x02									:	if(TM_DHT11_Read(&data)==TM_DHT11_OK) 
																	sprintf(str,"Temperature: %2.0fC\n\rHumidity: %2.0f%%\n\r",(float)data.Temp,(float)data.Hum);
																	//if(TM_USART_BufferEmpty(USART1)) TM_USART_Puts(USART1,str);
		 case	0x04									:	
		 case	osOK									:	break;
		 case	osFlagsErrorISR				:	
		 case	osFlagsErrorUnknown		:	
		 case	osFlagsErrorParameter	:	
		 case	osFlagsErrorResource	:	
		 case	osFlagsErrorTimeout		:	osEventFlagsClear(clk_id,osEventFlagsGet(clk_id));	break;
		}
	}
}

int main(void) 
{
/* Initialize system */
	SystemInit();
	
	/* Initialize delay */
	TM_DELAY_Init();

	/* Initialize RTC with internal 32.768kHz clock */
	/* It's very accurate XTAL=PC14 & PC15*/
	#ifdef ExternalSource
	//TM_RTC_Init(TM_RTC_ClockSource_External);
	if(!TM_RTC_Init(TM_RTC_ClockSource_External))
	#else
	/* Initialize RTC with internal 32.768kHz clock */
	/* It's not very accurate */
	//TM_RTC_Init(TM_RTC_ClockSource_Internal);
	if(!TM_RTC_Init(TM_RTC_ClockSource_Internal))
	#endif
	{
		/* RTC was first time initialized */
		/* Do your stuff here */
		/* eg. set default time */
		sscanf(Time,"%02d:%02d:%02d",&Cas.h,&Cas.m,&Cas.s);
		sscanf(Date,"%s %02d %4d",month,&Cas.day,&Cas.year);
		i=0;
		while(memcmp(month,Month[i++],sizeof("Jan"))!=0) Delay(10);	
		datatime.hours=(uint8_t)Cas.h;
		datatime.minutes=(uint8_t)Cas.m;
		datatime.seconds=(uint8_t)Cas.s;
		datatime.year=(uint8_t)(Cas.year-2000);
		datatime.date=(uint8_t)Cas.day;
		datatime.month=i;
		datatime.day=1;
		/* Set new time */
		TM_RTC_SetDateTime(&datatime,TM_RTC_Format_BIN);
	}
	
	//TM_RTC_Interrupts(TM_RTC_Int_125ms);
	//TM_RTC_Interrupts(TM_RTC_Int_500ms);
	TM_RTC_Interrupts(TM_RTC_Int_1s);
	//TM_RTC_Interrupts(TM_RTC_Int_2s);
	//TM_RTC_Interrupts(TM_RTC_Int_5s);

	/* Initialize Leds */
	// LED_GREEN PA7, LED_ORANGE PA6
	TM_DISCO_LedInit();

	/* Initialize button on board K0->PE4 K1->PE3 */
	TM_DISCO_ButtonInit();
	
	/* Initialize DAC channel 1,pin PA4 */
	TM_DAC_Init(TM_DAC1);
	/* Initialize DAC channel 2,pin PA5 */
	TM_DAC_Init(TM_DAC2);


	/* Initialize USART1, TX: PA9, RX: PA10 */
	TM_USART_Init(USART1, TM_USART_PinsPack_1, 115200);

	/* Initialize DHT11->PD1 */
	TM_DHT11_Init();
	
  // System Initialization
  SystemCoreClockUpdate();
#ifdef RTE_Compiler_EventRecorder
  // Initialize and start Event Recorder
  //EventRecorderInitialize(EventRecordDetail,1U);
	//EventRecorderInitialize(EventRecordAPI,1U);
	//EventRecorderInitialize(EventRecordOp,1U);
	EventRecorderInitialize(EventRecordError,1U);
	//EventRecorderInitialize(EventRecordAll,1U);
#endif
  osKernelInitialize();                 													// Initialize CMSIS-RTOS
	//param0=param1=param2,param3=param4=param5=param6=NULL;
	evt_id=osEventFlagsNew(NULL);																		// The function osEventFlagsNew creates a new event flags object that is used to send events across threads and returns the pointer to the event flags object identifier or NULL in case of an error. It can be safely called before the RTOS is started (call to osKernelStart), but not before it is initialized (call to osKernelInitialize).
	sig_id=osEventFlagsNew(NULL);																		// The function osEventFlagsNew creates a new event flags object that is used to send events across threads and returns the pointer to the event flags object identifier or NULL in case of an error. It can be safely called before the RTOS is started (call to osKernelStart), but not before it is initialized (call to osKernelInitialize).
	clk_id=osEventFlagsNew(NULL);																		// The function osEventFlagsNew creates a new event flags object that is used to send events across threads and returns the pointer to the event flags object identifier or NULL in case of an error. It can be safely called before the RTOS is started (call to osKernelStart), but not before it is initialized (call to osKernelInitialize).
	//int_id=osEventFlagsNew(NULL);																		// The function osEventFlagsNew creates a new event flags object that is used to send events across threads and returns the pointer to the event flags object identifier or NULL in case of an error. It can be safely called before the RTOS is started (call to osKernelStart), but not before it is initialized (call to osKernelInitialize).
	msgQueue=osMessageQueueNew(10,sizeof(msg_t),NULL);	  					// Create message queue for up to 10 messages of type msg_t

#ifdef SIMUL
	th0=osThreadNew(app_main,NULL,NULL);  													// Create application main thread - ide v RTOS simulatore !!!
	th1=osThreadNew(reg_k0,NULL,NULL);															// Create application main thread
	th2=osThreadNew(reg_v0,NULL,NULL);															// Create application main thread
	th3=osThreadNew(msg_app,NULL,NULL);      	    									// Create application main thread
	th4=osThreadNew(flag_app,NULL,NULL);      	    								// Create application main thread
	th5=osThreadNew(sig_app,NULL,NULL);  	    	    								// Create application main thread
	th6=osThreadNew(clk_app,NULL,NULL);  	    	    								// Create application main thread
#else
	th0=osThreadNew(app_main,&param0,&admin_app_attr);  						// Create application main thread - s parametrom &admin_app_attr nie je vidno v RTOS simulatore
	th1=osThreadNew(reg_k0,&param1,&reg_k0_attr);										// Create application main thread
	th2=osThreadNew(reg_v0,&param2,&reg_v0_attr);										// Create application main thread
	th3=osThreadNew(msg_app,&param3,&msg_app_attr);      	    			// Create application main thread
	th4=osThreadNew(flag_app,&param4,&flag_app_attr);      	    		// Create application main thread
	th5=osThreadNew(sig_app,&param5,&sig_app_attr);  	    	    		// Create application main thread
	th6=osThreadNew(clk_app,&param6,&clk_app_attr);  	    	    		// Create application main thread
#endif

  osKernelStart();                      													// Start thread execution
	
  for(;;) {}
}



